@@ -1,15 +1,18 @@
 from __future__ import annotations
 
 import re
+from contextlib import suppress
 from dataclasses import dataclass
 from math import ceil, exp, floor, isclose, isfinite, isinf, isnan, log, log10, modf
 from re import Match, search
 from typing import TYPE_CHECKING, Literal, assert_never, overload, override
 
 from utilities.errors import ImpossibleCaseError
+from utilities.re import ExtractGroupsError, extract_groups
 
 if TYPE_CHECKING:
-    from utilities.types import RoundMode
+    from utilities.types import MathRoundMode, Number, Sign
+
 
 MIN_FLOAT32, MAX_FLOAT32 = -3.4028234663852886e38, 3.4028234663852886e38
 MIN_FLOAT64, MAX_FLOAT64 = -1.7976931348623157e308, 1.7976931348623157e308
@@ -681,11 +684,32 @@ def order_of_magnitude(x: float, /, *, round_: bool = False) -> float:
 ##
 
 
+def parse_number(number: str, /) -> Number:
+    """Convert text into a number."""
+    with suppress(ValueError):
+        return int(number)
+    with suppress(ValueError):
+        return float(number)
+    raise ParseNumberError(number=number)
+
+
+@dataclass(kw_only=True, slots=True)
+class ParseNumberError(Exception):
+    number: str
+
+    @override
+    def __str__(self) -> str:
+        return f"Unable to parse number; got {self.number!r}"
+
+
+##
+
+
 def round_(
     x: float,
     /,
     *,
-    mode: RoundMode = "standard",
+    mode: MathRoundMode = "standard",
     rel_tol: float | None = None,
     abs_tol: float | None = None,
 ) -> int:
@@ -725,7 +749,7 @@ def round_(
 
 def _round_tie_standard(
     x: float,
-    mode: RoundMode,
+    mode: MathRoundMode,
     /,
     *,
     rel_tol: float | None = None,
@@ -734,9 +758,9 @@ def _round_tie_standard(
     """Round a float to an integer using the standard method."""
     frac, _ = modf(x)
     if _is_close(abs(frac), 0.5, rel_tol=rel_tol, abs_tol=abs_tol):
-        mode_use: RoundMode = mode
+        mode_use: MathRoundMode = mode
     else:
-        mode_use: RoundMode = "standard"
+        mode_use: MathRoundMode = "standard"
     return round_(x, mode=mode_use)
 
 
@@ -752,9 +776,9 @@ def round_float_imprecisions(
 ) -> float:
     """Round a float, removing binary representation imprecisions."""
     try:
-        ((head, tail),) = _ROUND_FLOAT_IMPRECISIONS_PATTERN.findall(str(x))
-    except ValueError:
-        ((head, tail),) = _ROUND_FLOAT_IMPRECISIONS_PATTERN.findall(f"{x:.20f}")
+        head, tail = extract_groups(_ROUND_FLOAT_IMPRECISIONS_PATTERN, str(x))
+    except ExtractGroupsError:
+        head, tail = extract_groups(_ROUND_FLOAT_IMPRECISIONS_PATTERN, f"{x:.20f}")
     half = ceil(decimals / 2)
     pattern0 = search(rf"^([0-9]+?)(0{{{half},}})([0-9]+?)$", tail)
     pattern9 = search(rf"^(0*)([0-9]+?)(9{{{half},}})([0-9]+?)$", tail)
@@ -800,7 +824,7 @@ def round_to_float(
     y: float,
     /,
     *,
-    mode: RoundMode = "standard",
+    mode: MathRoundMode = "standard",
     rel_tol: float | None = None,
     abs_tol: float | None = None,
 ) -> float:
@@ -837,7 +861,7 @@ class SafeRoundError(Exception):
 
 def sign(
     x: float, /, *, rel_tol: float | None = None, abs_tol: float | None = None
-) -> Literal[-1, 0, 1]:
+) -> Sign:
     """Get the sign of an integer/float."""
     match x:
         case int():
@@ -887,6 +911,7 @@ __all__ = [
     "MIN_UINT64",
     "CheckIntegerError",
     "EWMParametersError",
+    "ParseNumberError",
     "SafeRoundError",
     "check_integer",
     "ewm_parameters",
@@ -934,6 +959,7 @@ __all__ = [
     "is_zero_or_non_micro_or_nan",
     "number_of_decimals",
     "order_of_magnitude",
+    "parse_number",
     "round_",
     "round_float_imprecisions",
     "round_to_float",
