@@ -109,7 +109,9 @@ class CoroutineLooper:
                 message="coroutine '.*' was never awaited", category=RuntimeWarning
             ):
                 del coro
-            get_logger(logger=self.logger).error("Error running %r", name)
+            get_logger(logger=self.logger).error(
+                "Error running %r", name, exc_info=True
+            )
 
 
 ##
@@ -173,7 +175,9 @@ async def _get_first_available_lock(
     )
     async with timeout_td(timeout, error=error):  # skipif-ci-and-not-linux
         while True:
-            if (result := await _get_first_available_lock_if_any(locks)) is not None:
+            if (
+                (result := await _get_first_available_lock_if_any(locks)) is not None
+            ) and (await result.locked() > 0.0):
                 return result
             await sleep_td(sleep)
 
