@@ -6,6 +6,8 @@ from dataclasses import dataclass
 from functools import partial
 from getpass import getuser
 from itertools import repeat
+from logging import exception
+from os import getpid
 from pathlib import Path
 from socket import gethostname
 from traceback import TracebackException
@@ -25,6 +27,7 @@ from utilities.reprlib import (
     RICH_MAX_WIDTH,
     yield_mapping_repr,
 )
+from utilities.tzlocal import LOCAL_TIME_ZONE_NAME
 from utilities.version import get_version
 from utilities.whenever import (
     format_compact,
@@ -96,16 +99,20 @@ def _yield_header_lines(
     """Yield the header lines."""
     now = get_now_local()
     start_use = to_zoned_date_time(date_time=start)
-    yield f"Date/time | {format_compact(now)}"
-    start_str = "" if start_use is None else format_compact(start_use)
-    yield f"Started   | {start_str}"
+    yield f"Date/time  | {format_compact(now)}"
+    if start_use is None:
+        start_str = ""
+    else:
+        start_str = format_compact(start_use.to_tz(LOCAL_TIME_ZONE_NAME))
+    yield f"Started    | {start_str}"
     delta = None if start_use is None else (now - start_use)
     delta_str = "" if delta is None else delta.format_common_iso()
-    yield f"Duration  | {delta_str}"
-    yield f"User      | {getuser()}"
-    yield f"Host      | {gethostname()}"
+    yield f"Duration   | {delta_str}"
+    yield f"User       | {getuser()}"
+    yield f"Host       | {gethostname()}"
+    yield f"Process ID | {getpid()}"
     version_use = "" if version is None else get_version(version=version)
-    yield f"Version   | {version_use}"
+    yield f"Version    | {version_use}"
     yield ""
 
 
@@ -275,9 +282,14 @@ def _make_except_hook_inner(
         with writer(path, overwrite=True) as temp:
             _ = temp.write_text(full)
     if slack_url is not None:  # pragma: no cover
-        from utilities.slack_sdk import send_to_slack
+        from utilities.slack_sdk import SendToSlackError, send_to_slack
+
+        try:
+            send_to_slack(slack_url, f"```{slim}```")
+        except SendToSlackError as error:
+            msg = str(error)
+            exception(msg)  # noqa: LOG015
 
-        send_to_slack(slack_url, f"```{slim}```")
     if to_bool(bool_=pudb):  # pragma: no cover
         from pudb import post_mortem
 
